;;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: COMMON-LISP-USER -*-
;;;;
;;;; Written by Patrick W. Yaner <yaner@cc.gatech.edu>
;;;;   and Jim Davies <jim@jimdavies.org>
;;;;
;;;; Copyright (c) 2003
;;;; Georgia Institute of Technology
;;;; All rights reserved

(in-package :common-lisp-user)

;;;;;;;;; Description
;;;
;;; Frame system (memory file, essentially) representing the shape
;;; type hierarchy and the three examples for the visual abstraction
;;; mapping program.
;;;
;;; Dependencies:
;;; - frames
;;; - index

(clear-memory)

;;;;;;;;; Shape type hierarchy

(defframe point (shape))
(defframe curve (shape))
(defframe open-curve (curve))
(defframe spline (open-curve))
(defframe segmented-line (open-curve))
(defframe line (segmented-line))
(defframe close-curve (curve))
(defframe ellipse (closed-curve))
(defframe circle (ellipse))
(defframe polygon (closed-curve))
(defframe quadrilateral (polygon))
(defframe parallelogram (quadrilateral))
(defframe rectangle (parallelogram))
(defframe square (rectangle))
(defframe triangle (polygon))
(defframe right-triangle (triangle))
(defframe isosceles-triangle (triangle))
(defframe equilateral-triangle (isosceles-triangle))
(defframe scalene-triangle (triangle))

(defframe ^simage (^index))

(defframe ^visual-element (^index))

;;;;;;;;;; Face example

;; target
(defframe t-face-simage (^simage)
  (contains-elements (t-face-head
		      t-face-eye
		      t-face-nose
		      t-face-mouth)))

(defframe t-face-head (^visual-element)
  (in-simage  t-face-simage))
  (looks-like ellipse)
  (has-size   large))

(defframe t-face-eye (^visual-element)
  (in-simage  t-face-simage)
  (looks-like circle)
  (has-size   small))

(defframe t-face-nose (^visual-element)
  (in-simage  t-face-simage)
  (looks-like right-triangle)
  (has-size   small))

(defframe t-face-mouth (^visual-element)
  (in-simage  t-face-simage)
  (looks-like line)
  (has-size   small))

;; source
(defframe s-face-simage (^simage)
  (contains-elements (s-face-head
		      s-face-eye
		      s-face-nose
		      s-face-mouth)))

(defframe s-face-head (^visual-element)
  (in-simage  s-face-simage))
  (looks-like ellipse)
  (has-size   large))

(defframe s-face-eye (^visual-element)
  (in-simage  s-face-simage)
  (looks-like ellipse)
  (has-size   small))

(defframe s-face-nose (^visual-element)
  (in-simage  s-face-simage)
  (looks-like quadrilateral)
  (has-size   small))

(defframe s-face-mouth (^visual-element)
  (in-simage  s-face-simage)
  (looks-like segmented-line)
  (has-size   small))

;;;;;;;;;; Table example

;; target
(defframe t-table-simage (^simage)
  (contains-elements (t-butterdish
		      t-glass
		      t-fork-aggregate
		      t-spoon-aggregate
		      t-knife
		      t-plate
		      t-napkin)

(defframe t-butterdish (^visual-element)
  (in-simage  t-table-simage)
  (looks-like rectangle)
  (has-size   small))

(defframe t-glass (^visual-element)
  (in-simage  t-table-simage)
  (looks-like circle)
  (has-size   small)

(defframe t-fork-aggregate (^visual-element)
  (in-simage      t-table-simage)
  (looks-like     aggregate)
  (has-components (t-fork-tines
		   t-fork-shaft)))

(defframe t-fork-tines (^visual-element)
  (in-simage    t-table-simage)
  (looks-like   square)
  (has-size     small)
  (component-of t-fork-aggregate)

(defframe t-fork-shaft (^visual-element)
  (in-simage    t-table-simage)
  (looks-like   line)
  (has-size     small)
  (component-of t-fork-aggregate)

(defframe t-spoon-aggregate (^visual-element)
  (in-simage      t-table-simage)
  (looks-like     aggregate)
  (has-components (t-spoon-bowl
		   t-spoon-shaft)))

(defframe t-spoon-bowl (^visual-element)
  (in-siamge    t-table-simage)
  (looks-like   circle)
  (component-of t-spoon-aggregate)
  (has-size     small))

(defframe t-knife (^visual-element)
  (in-simage  t-table-simage)
  (looks-like line)
  (has-size   small))

(defframe t-plate (^visual-element)
  (in-simage  t-table-simage)
  (looks-like ellipse)
  (has-size   medium))

(defframe t-napkin (^visual-element)
  (in-simage  t-table-simage)
  (looks-like right-triangle)
  (has-size   small))

;; source
(defframe s-table-simage (^simage)
  (contains-elements (s-butterdish
		      s-glass
		      s-fork-aggregate
		      s-spoon-aggregate
		      s-knife
		      s-plate
		      s-napkin)

(defframe s-butterdish (^visual-element)
  (in-simage  s-table-simage)
  (looks-like rectangle)
  (has-size   small))

(defframe s-glass (^visual-element)
  (in-simage  s-table-simage)
  (looks-like circle)
  (has-size   small)

(defframe s-fork-aggregate (^visual-element)
  (in-simage      s-table-simage)
  (looks-like     aggregate)
  (has-components (s-fork-tines
		   s-fork-shaft)))

(defframe s-fork-tines (^visual-element)
  (in-simage    s-table-simage)
  (looks-like   square)
  (has-size     small)
  (component-of s-fork-aggregate)

(defframe s-fork-shaft (^visual-element)
  (in-simage    s-table-simage)
  (looks-like   line)
  (has-size     small)
  (component-of s-fork-aggregate)

(defframe s-spoon-aggregate (^visual-element)
  (in-simage      s-table-simage)
  (looks-like     aggregate)
  (has-components (s-spoon-bowl
		   s-spoon-shaft)))

(defframe s-spoon-bowl (^visual-element)
  (in-siamge    s-table-simage)
  (looks-like   circle)
  (component-of s-spoon-aggregate)
  (has-size     small))

(defframe s-knife (^visual-element)
  (in-simage  s-table-simage)
  (looks-like line)
  (has-size   small))

(defframe s-plate (^visual-element)
  (in-simage  s-table-simage)
  (looks-like circle)
  (has-size   medium))

(defframe s-napkin (^visual-element)
  (in-simage  s-table-simage)
  (looks-like rectangle)
  (has-size   small))

;;;;;;;;;; Parking lot example

;; target
(defframe t-lot-simage (^simage)
  (contains-elements (t-puddle
		      t-truck-aggregate
		      t-dumpster
		      t-cans)))

(defframe t-puddle (^visual-element)
  (in-simage  t-lot-simage)
  (looks-like closed-curve)
  (has-size   large)

(defframe t-truck-aggregate (^visual-element)
  (in-simage      t-lot-simage)
  (looks-like     aggregate)
  (has-components (t-rig
		   t-trailer)))

(defframe t-rig (^visual-element)
  (in-simage    t-lot-simage)
  (looks-like   square)
  (has-size     small)
  (component-of t-truck-aggregate))

(defframe t-trailer (^visual-element)
  (in-simage    t-lot-simage)
  (looks-like   retangle)
  (has-size     medium)
  (component-of t-truck-aggregate))

(defframe t-dumpster (^visual-element)
  (in-simage  t-lot-simage)
  (looks-like rectangle)
  (has-size   small))

(defframe t-cans (^visual-element)
  (in-simage   t-lot-simage)
  (looks-like  set)
  (has-members (t-can1 t-can2 t-can3 t-can4 t-can5)))

(defframe t-can1 (^visual-element)
  (in-simage  t-lot-simage)
  (looks-like circle)
  (in-set     t-cans)
  (has-size   small))

(defframe t-can2 (^visual-element)
  (in-simage  t-lot-simage)
  (looks-like circle)
  (in-set     t-cans)
  (has-size   small))

(defframe t-can3 (^visual-element)
  (in-simage  t-lot-simage)
  (looks-like circle)
  (in-set     t-cans)
  (has-size   small))

(defframe t-can4 (^visual-element)
  (in-simage  t-lot-simage)
  (looks-like circle)
  (in-set     t-cans)
  (has-size   small))

(defframe t-can4 (^visual-element)
  (in-simage  t-lot-simage)
  (looks-like circle)
  (in-set     t-cans)
  (has-size   small))

;; source
(defframe s-lot-simage (^simage)
  (contains-elements (s-puddle
		      s-truck-aggregate
		      s-dumpster
		      s-cans)))

(defframe s-puddle (^visual-element)
  (in-simage  s-lot-simage)
  (looks-like closed-curve)
  (has-size   large)

(defframe s-truck-aggregate (^visual-element)
  (in-simage      s-lot-simage)
  (looks-like     aggregate)
  (has-components (s-rig
		   s-trailer)))

(defframe s-rig (^visual-element)
  (in-simage    s-lot-simage)
  (looks-like   square)
  (has-size     small)
  (component-of s-truck-aggregate))

(defframe s-trailer (^visual-element)
  (in-simage    s-lot-simage)
  (looks-like   retangle)
  (has-size     medium)
  (component-of s-truck-aggregate))

(defframe s-dumpster (^visual-element)
  (in-simage  s-lot-simage)
  (looks-like rectangle)
  (has-size   small))

(defframe s-cans (^visual-element)
  (in-simage   s-lot-simage)
  (looks-like  set)
  (has-members (s-can1 s-can2 s-can3)))

(defframe s-can1 (^visual-element)
  (in-simage  s-lot-simage)
  (looks-like circle)
  (in-set     s-cans)
  (has-size   small))

(defframe s-can2 (^visual-element)
  (in-simage  s-lot-simage)
  (looks-like circle)
  (in-set     s-cans)
  (has-size   small))

(defframe s-can3 (^visual-element)
  (in-simage  s-lot-simage)
  (looks-like circle)
  (in-set     s-cans)
  (has-size   small))

(defframe s-can4 (^visual-element)
  (in-simage  s-lot-simage)
  (looks-like circle)
  (in-set     s-cans)
  (has-size   small))

(defframe s-can4 (^visual-element)
  (in-simage  s-lot-simage)
  (looks-like circle)
  (in-set     s-cans)
  (has-size   small))
